@@ -3,9 +3,9 @@
 ;; Copyright (C) 2025 Shariff AM Faleel
 
 ;; Author: Shariff AM Faleel
-;; Package-Requires: ((emacs "28") (org-roam "2.2.0") (s "1.12.0") (magit-section "3.3.0") (transient "0.4") (org-super-agenda "1.2") (dash "2.0"))
-;; Package-Version: 20251116.829
-;; Package-Revision: b1b91d6e1847
+;; Package-Requires: ((emacs "28") (org-roam "2.2.0") (s "1.12.0") (magit-section "3.3.0") (transient "0.4") (dash "2.0"))
+;; Package-Version: 20251217.2009
+;; Package-Revision: 420ce9b5a28c
 ;; Homepage: https://github.com/ahmed-shariff/org-roam-ql
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -30,9 +30,9 @@
 
 (require 'magit-section)
 (require 'transient)
-(require 'org-super-agenda)
 (require 'org-roam-utils)
 (require 'org-roam-node)
+(require 'org-agenda)
 (require 'dash)
 (require 's)
 (require 'bookmark)
@@ -1006,14 +1006,13 @@ Similar to `org-roam-mode', but doesn't default to the
                               title)))
     (org-roam-ql--refresh-buffer-with-display-function #'org-roam-ql--roam-buffer-for-nodes)))
 
-(defun org-roam-ql--render-roam-buffer (sections title buffer-name source-or-query sort-fn preview-fn)
+(defun org-roam-ql--render-roam-buffer (sections title buffer-name source-or-query sort-fn pre-render-fn post-render-fn)
   "Render SECTIONS (list of functions) in an org-roam-ql buffer.
-TITLE is the title, BUFFER-NAME is the name for the buffer.
-See `org-roam-nodes' for information on SOURCE-OR-QUERY.
-See `org-roam-ql-nodes' for SORT-FN.
-See `org-roam-ql-search' for PREVIEW-FN.
-When all sections are inserted, will call hook
-`org-roam-ql-buffer-postrender-functions'."
+TITLE is the title, BUFFER-NAME is the name for the buffer.  See
+`org-roam-nodes' for information on SOURCE-OR-QUERY.  See
+`org-roam-ql-nodes' for SORT-FN.  PRE-RENDER-FN, if non-nil, will be
+called before rendering all SECTIONS.  POST-RENDER-FN, if non-nil,
+will be called after rendering all SECTIONS. "
   ;; copied  from `org-roam-buffer-render-contents'
   (with-current-buffer (get-buffer-create buffer-name)
     (let ((inhibit-read-only t))
@@ -1026,13 +1025,14 @@ When all sections are inserted, will call hook
             org-roam-ql-buffer-title title
             org-roam-ql-buffer-sort sort-fn
             org-roam-ql-buffer-in "org-roam-db")
-      (when preview-fn
-        (setq-local org-roam-ql-preview-function preview-fn))
+      (when pre-render-fn
+        (funcall pre-render-fn))
       (magit-insert-section section (org-roam)
          (magit-insert-heading)
          (dolist (section sections)
            (funcall section)))
-      (run-hooks 'org-roam-ql-buffer-postrender-functions)
+      (when post-render-fn
+        (funcall post-render-fn))
       (goto-char 0))
     (display-buffer (current-buffer))))
 
@@ -1083,7 +1083,12 @@ See `org-roam-ql-search' for PREVIEW-FN."
   (org-roam-ql--render-roam-buffer
    (list
     (org-roam-ql--nodes-section nodes "Nodes:"))
-   title buffer-name (or source-or-query nodes) sort-fn preview-fn))
+   title buffer-name (or source-or-query nodes) sort-fn
+   (lambda ()
+     (when preview-fn
+       (setq-local org-roam-ql-preview-function preview-fn)))
+   (lambda ()
+     (run-hooks 'org-roam-ql-buffer-postrender-functions))))
 
 (defun org-roam-ql--nodes-from-roam-buffer (buffer)
   "Collect the org-roam-nodes from a valid BUFFER."
@@ -1157,12 +1162,13 @@ See docs of `bookmark-make-record-function'."
 Based on `org-agenda-mode-map'.")
 
 ;; KLUDGE: preview-fn is left _
-(defun org-roam-ql--agenda-buffer-for-nodes (nodes title buffer-name &optional source-or-query sort-fn _ super-groups)
+(defun org-roam-ql--agenda-buffer-for-nodes (nodes title buffer-name &optional source-or-query sort-fn _ process-entries-fn)
   "Display the nodes in a agenda like buffer.
 See `org-roam-ql--render-roam-buffer' for TITLE BUFFER-NAME and SOURCE-OR-QUERY.
 See `org-roam-ql--nodes-section' for NODES.
 See `org-roam-ql-nodes' for SORT-FN.
-See `org-super-agenda' for SUPER-GROUPS."
+`process-entries-fn' would recieve a list of strings, each an entry in
+the agenda buffer and return the list of strings to be displayed."
   (with-current-buffer (get-buffer-create buffer-name)
     (unless (eq major-mode 'org-agenda-mode)
       (org-agenda-mode)
@@ -1186,11 +1192,8 @@ See `org-super-agenda' for SUPER-GROUPS."
         (dolist-with-progress-reporter (node nodes)
             (format "Processing %s nodes" (length nodes))
           (push (org-roam-ql-view--format-node node) strings))
-        (when super-groups
-          (let ((org-super-agenda-groups (cl-etypecase super-groups
-                                           (symbol (symbol-value super-groups))
-                                           (list super-groups))))
-            (setf strings (org-super-agenda--group-items strings))))
+        (when process-entries-fn
+          (setf strings (funcall process-entries-fn strings)))
         (insert (string-join strings "\n") "\n")
         (pop-to-buffer (current-buffer))
         (org-agenda-finalize)
@@ -1306,9 +1309,41 @@ If there are entries that do not have an ID, it will signal an error"
 ;; `org-roam' sections (`org-roam-mode-sections') that can also be
 ;; filtered with org-roam-ql
 ;; *****************************************************************************
-(cl-defun org-roam-backlinks-section-with-ql-filter (node &key (unique nil) (show-backlink-p nil)
-                                                          (section-heading "Backlinks:"))
-  "Same as `org-roam-backlinks-section', but considers the filter (`org-roam-ql--filter-for-roam') of the current buffer.
+(cl-defun org-roam-ql-backlinks-get (query &key unique)
+  "Return the backlinks for any node in query.
+
+ When UNIQUE is nil, show all positions where references are found.
+ When UNIQUE is t, limit to unique sources."
+  ;; Copied from `org-roam-backlinks-get'
+  (let* ((sql (if unique
+                  [:select :distinct [source dest pos properties]
+                   :from links
+                   :where (in dest $v1)
+                   :and (= type "id")
+                   :group :by source
+                   :having (funcall min pos)]
+                [:select [source dest pos properties]
+                 :from links
+                 :where (in dest $v1)
+                 :and (= type "id")]))
+         (backlinks (org-roam-db-query sql (apply #'vector
+                                                  (-map #'org-roam-node-id (org-roam-ql-nodes query))))))
+    (cl-loop for backlink in backlinks
+             collect (pcase-let ((`(,source-id ,dest-id ,pos ,properties) backlink))
+                       (org-roam-populate
+                        (org-roam-backlink-create
+                         :source-node (org-roam-node-create :id source-id)
+                         :target-node (org-roam-node-create :id dest-id)
+                         :point pos
+                         :properties properties))))))
+
+(cl-defun org-roam-backlinks-section-with-ql-filter (node-or-query &key (unique nil) (show-backlink-p nil)
+                                                                   (section-heading "Backlinks:"))
+  "Render section for backlinks to NODE-OR-QUERY.
+
+This is Same as `org-roam-backlinks-section', but can accept a query
+instead of a node and considers the filter
+(`org-roam-ql--filter-for-roam') of the current buffer.
 
 When UNIQUE is nil, show all positions where references are found.
 When UNIQUE is t, limit to unique sources.
@@ -1317,7 +1352,10 @@ When SHOW-BACKLINK-P is not null, only show backlinks for which
 this predicate is not nil.
 
 SECTION-HEADING is the string used as a heading for the backlink section."
-  (let* ((backlinks (seq-sort #'org-roam-backlinks-sort (org-roam-backlinks-get node :unique unique)))
+  (let* ((backlinks (seq-sort #'org-roam-backlinks-sort
+                              (if (org-roam-node-p node-or-query)
+                                  (org-roam-backlinks-get node-or-query :unique unique)
+                                (org-roam-ql-backlinks-get node-or-query :unique unique))))
          (filter org-roam-ql--filter-for-roam)
          (filter-node-ids (and filter
                                (condition-case _err
@@ -1350,12 +1388,43 @@ SECTION-HEADING is the string used as a heading for the backlink section."
              :properties (org-roam-backlink-properties backlink))))
         (insert ?\n)))))
 
-(defun org-roam-reflinks-section-with-ql-filter (node)
-  "The reflinks section for NODE with org-roam-ql filter (`org-roam-ql--filter-for-roam').
+(defun org-roam-ql-reflinks-get (query)
+  "Return the reflinks for any node in query."
+  (let ((refs (org-roam-db-query [:select :distinct [refs:ref links:source links:pos links:properties]
+                                  :from refs
+                                  :left-join links
+                                  :where (in refs:node-id $v1)
+                                  :and (= links:dest refs:ref)
+                                  :union
+                                  :select :distinct [refs:ref citations:node-id
+                                                     citations:pos citations:properties]
+                                  :from refs
+                                  :left-join citations
+                                  :where (in refs:node-id $v1)
+                                  :and (= citations:cite-key refs:ref)]
+                                 (apply #'vector
+                                        (-map #'org-roam-node-id (org-roam-ql-nodes query)))))
+        links)
+    (pcase-dolist (`(,ref ,source-id ,pos ,properties) refs)
+      (push (org-roam-populate
+             (org-roam-reflink-create
+              :source-node (org-roam-node-create :id source-id)
+              :ref ref
+              :point pos
+              :properties properties)) links))
+    links))
 
-Same as `org-roam-reflinks-section'."
-  (when-let ((refs (org-roam-node-refs node))
-             (reflinks (seq-sort #'org-roam-reflinks-sort (org-roam-reflinks-get node))))
+(defun org-roam-reflinks-section-with-ql-filter (node-or-query)
+  "The reflinks section for NODE-OR-QUERY with org-roam-ql filter (`org-roam-ql--filter-for-roam').
+
+Same as `org-roam-reflinks-section', but will take both node or a query."
+  (when-let ((refs (if (org-roam-node-p node-or-query)
+                       (org-roam-node-refs node-or-query)
+                     (-any-p #'org-roam-node-refs (org-roam-ql-nodes node-or-query))))
+             (reflinks (seq-sort #'org-roam-reflinks-sort
+                                 (if (org-roam-node-p node-or-query)
+                                     (org-roam-reflinks-get node-or-query)
+                                   (org-roam-ql-reflinks-get node-or-query)))))
     (let* ((filter org-roam-ql--filter-for-roam)
            (filter-node-ids (and filter
                                  (condition-case _err
